﻿using Newtonsoft.Json;
using RestSharp;
using System;
using System.Net;
using System.Xml;

namespace SignNow
{
    public class OAuth2
    {
        /// <summary>
        /// Request an Access Token using the User's Credentials
        /// </summary>
        /// <param name="Email"></param>
        /// <param name="Password"></param>
        /// <param name="Scope">A space delimited list of API URIs e.g. "user%20documents%20user%2Fdocumentsv2"</param>
        /// <param name="ResultFormat">JSON, XML</param>
        /// <returns>New Access Token, Token Type, Expires In, Refresh Token, ID, Scope</returns>
        public static dynamic RequestToken(string Email, string Password, string Scope = "*", string ResultFormat = "JSON")
        {
            var client = new RestClient();
            client.BaseUrl = new Uri(Config.ApiHost);

            System.Net.ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;

            var request = new RestRequest("/oauth2/token", Method.POST)
                .AddHeader("Accept", "application/json")
                .AddHeader("Authorization", "Basic " + Config.EncodedClientCredentials)
                .AddHeader("Content-Type", "application/x-www-form-urlencoded")
                .AddParameter("username", Email)
                .AddParameter("password", Password)
                .AddParameter("grant_type", "password")
                .AddParameter("scope", Scope);

                request.RequestFormat = DataFormat.Json;

            var response = client.Execute(request);

            dynamic results = "";

            if (response.StatusCode == HttpStatusCode.OK)
            {
                results = response.Content;
            }
            else
            {
                Console.WriteLine(response.Content.ToString());
                results = response.Content.ToString();
            }

            if (ResultFormat == "JSON")
            {
                results = JsonConvert.DeserializeObject(results);
            }
            else if (ResultFormat == "XML")
            {
                results = (XmlDocument)JsonConvert.DeserializeXmlNode(results, "root");
            }

            return results;
        }

        /// <summary>
        /// Request an Access Token using the User's Credentials
        /// </summary>
        /// <param name="Email"></param>
        /// <param name="Password"></param>
        /// <param name="documentId"></param>
        /// <param name="ResultFormat">JSON, XML</param>
        /// <returns>New Access Token, Token Type, Expires In, Refresh Token, ID, For Limited Scope access to documentId</returns>
        public static dynamic RequestTokenLimitedScopeDocument(string Email, string Password, string documentId, string ResultFormat = "JSON")
        {
            var client = new RestClient();
            client.BaseUrl = new Uri(Config.ApiHost);

            var request = new RestRequest("/oauth2/token", Method.POST)
                .AddHeader("Accept", "application/json")
                .AddHeader("Authorization", "Basic " + Config.EncodedClientCredentials)
                .AddHeader("Content-Type", "application/x-www-form-urlencoded");
            /*.AddParameter("username", Email)
            .AddParameter("password", Password)
            .AddParameter("grant_type", "password")
            .AddParameter("scope", "signer_limited_scope_token")*/
            //The above does not work because they want it in a very special format
            //See Part 2, Step 3 to see what I mean. https://help.signnow.com/v1.0/docs/create-a-signing-session-invite-with-fields 
            //paramater=value&param2=value2[SPACE]document/documentid anyways, I had to code it as seen on the next line for it to work. JAH 12/4/2018
            string data = $"username={WebUtility.UrlEncode(Email)}&password={WebUtility.UrlEncode(Password)}&grant_type=password&scope=signer_limited_scope_token document/{documentId}";
            
            request.AddParameter("application/x-www-form-urlencoded", data, ParameterType.RequestBody);

            request.RequestFormat = DataFormat.Json;

            var response = client.Execute(request);

            dynamic results = "";

            if (response.StatusCode == HttpStatusCode.OK)
            {
                results = response.Content;
            }
            else
            {
                Console.WriteLine(response.Content.ToString());
                results = response.Content.ToString();
            }

            if (ResultFormat == "JSON")
            {
                results = JsonConvert.DeserializeObject(results);
            }
            else if (ResultFormat == "XML")
            {
                results = (XmlDocument)JsonConvert.DeserializeXmlNode(results, "root");
            }

            return results;
        }

        /// <summary>
        /// Verify a User's Access Token
        /// </summary>
        /// <param name="AccessToken">User's Access Token</param>
        /// <param name="ResultFormat">JSON, XML</param>
        /// <returns>Access Token, Token Type, Expires In, Refresh Token, Scope</returns>
        public static dynamic Verify(string AccessToken, string ResultFormat = "JSON")
        {
            var client = new RestClient();
            client.BaseUrl = new Uri(Config.ApiHost);

            var request = new RestRequest("/oauth2/token", Method.GET)
                .AddHeader("Accept", "application/json")
                .AddHeader("Authorization", "Bearer " + AccessToken);

            var response = client.Execute(request);

            dynamic results = "";

            if (response.StatusCode == HttpStatusCode.OK)
            {
                results = response.Content;
            }
            else
            {
                Console.WriteLine(response.Content.ToString());
                results = response.Content.ToString();
            }

            if (ResultFormat == "JSON")
            {
                results = JsonConvert.DeserializeObject(results);
            }
            else if (ResultFormat == "XML")
            {
                results = (XmlDocument)JsonConvert.DeserializeXmlNode(results, "root");
            }

            return results;
        }
    }
}
